/*
Copyright (c) 2004, Marjan Punik, Mate Betek, Bla Kurent

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
* Neither the name of the Faculty of Computer and Information Science 
nor the names of its contributors may be used to endorse or promote
products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/
using System;
using System.Collections;
using System.Drawing;


namespace symmetry
{
	/// <summary>
	/// Summary description for Strokes1.
	/// </summary>
	/*-------------------------------------------------------------------------------------*/
	
	
	
	//Razred v katerega se shranjujejo vse poteze 
	public class Strokes1
	{
		private ArrayList allStrokes = new ArrayList();
		

		public Strokes1()
		{
			//
			// TODO: Add constructor logic here
			//
			lock(this)
			{
				allStrokes.Clear();
			}
		}

		~Strokes1()
		{
			lock(this)
			{
				allStrokes.Clear();
			}
		}

	
		public  void addStroke(Stroke1 s)
		{
			lock(this)
			{
				allStrokes.Add(s);
			}
		
		}

		public int numberOfStrokes()
		{


			lock(this)
			{
				return allStrokes.Count;
			}

		}
		
		public Stroke1 returnStrokeObject(int i)
		{
			lock(this)
			{
				return (Stroke1)allStrokes[i];
			}
		}

		public void clearStrokes()
		{
			lock(this)
			{
				int i;
				Stroke1 st;
				for(i = 0;i < allStrokes.Count;i++)
				{
					st = (Stroke1)allStrokes[i];
					st.clearStroke();
			
				}
				allStrokes.Clear();
			}

		}

		public void clearLastStroke()
		{
			lock(this)
			{
				int i;
				i = allStrokes.Count;
				clearOneStroke(i-1);
			}

		}

		public void clearOneStroke(int i)
		{
			lock(this)
			{
				Stroke1 st;
	
				if(i >= 0)
				{
					st = (Stroke1)allStrokes[i];
					st.clearStroke();
					allStrokes.RemoveAt(i);
				}
			}
		}

    }
	/*----------------------------------------------------------------------------------------------*/
	//razred v katerega se shrani ena poteza z Pen-om
	public class Stroke1
	{
		
		private ArrayList oneStroke = new ArrayList();
		private ArrayList time_intervals = new ArrayList();
		public  ArrayList pointRecognition = new ArrayList();  //ima atribut 'public', ker ta seznam spreminjamo pri sprotnem racunanju regresije
		private ArrayList pointRecognitionAngle = new ArrayList();  //seznam kotov
		private ArrayList pointRecognitionSpeed = new ArrayList(); //seznam za hitrost
		private ArrayList pointRecognitionAcc = new ArrayList(); //seznam pospekov
		private ArrayList pointRecognitionAll = new ArrayList(); //seznam zdruenih atributov
		private ArrayList pointForce  = new ArrayList();		//sem shranjujemo posamezne lastnosti tock za Sile
		private ForceSimulator forceSim = new ForceSimulator();	//simulator delovanja vseh sil na tocke
		
		private Color strokeColor = new Color();
		private int width;
		private Point tockaK= new Point();  //tocka kota po metodi najvecje oddaljenosti od diagonale
		private bool sklenjenObjekt = false;
		public ArrayList markedCornersL = new ArrayList();
		public bool isConnected=false;
	    //public ArrayList energy = new ArrayList(); //vsaka tocka ima svojo energijo
		//public ArrayList currentForce = new ArrayList(); //sila za vsako tocko

		public Point cPoint = new Point();
		public double radius;
		public bool isCircle = false;
		public ArrayList tempMCL = new ArrayList(); //pomozni seznam za izris

		public ArrayList imenPovezav = new ArrayList();
        public ArrayList inkiPovezav = new ArrayList();
		public ArrayList vzporednePovezave = new ArrayList();
		public ArrayList sosednjiStroki = new ArrayList();


		public Stroke1()
		{
			//
			// TODO: Add constructor logic here
			//
			lock(this)
			{
				oneStroke.Clear();
			}

		}

		~Stroke1()
		{
			lock(this)
			{
				oneStroke.Clear();
			}
		}
		
		public void addPoint(Point p,double tick)		//dodamo tocko na konec seznama
		{
			lock(this)
			{
				oneStroke.Add(p);
				time_intervals.Add(tick);

			}
	    }

		public void insertPoint(Point p,double tick,int index)	//dodamo tocko na dolocen indeks v seznamu
		{
			lock(this)
			{
			
				oneStroke.Insert(index,p);
				time_intervals.Add(tick);
			}
		}

		public void addPoint(Point p,double tick,double recognition)	//dodamo tocko na konec seznama
		{
			lock(this)
			{
				oneStroke.Add(p);
				time_intervals.Add(tick);
				pointRecognition.Add(recognition);
			}
		}

		public void changePoint(Point p,int index)		//spremenimo kordinate tocke na mestu index
		{
			lock(this)
			{
				
				oneStroke.RemoveAt(index);
				oneStroke.Insert(index,p);
			}
			
		}

		  
	
		public void addRecognition(double recognition)		//dodamo prepoznavo
		{
			lock(this)
			{
				pointRecognition.Add(recognition);
			}
	
		}
		//------------------------------------------------------------------
		public void addRecognitionAngle(double angle)  //dodamo kot
		{
			lock(this)
			{
				pointRecognitionAngle.Add(angle);
			}
		}
		public void ChangeRecognitionAngle(double angle, int index) //spremenimo vrednost v seznamu kotov
		{
			lock(this)
			{
				pointRecognitionAngle[index]=angle;
			}
		}
		public void ChangeRecognition(double rec, int i) //spremenimo vrednost v seznamu vrednosti lin. r.
		{
			lock(this)
			{
				pointRecognition[i]=rec;
			}
		}
		public void addRecognitionAll(double d)
		{
			lock(this)
			{
				pointRecognitionAll.Add(d);
			}
		}
		public void ChangeRecognitionAll(double d, int i)
		{
			lock(this)
			{
				pointRecognitionAll[i]=d;
			}
		}

		public double returnRecognitionAngle(int i)
		{
			lock(this)
			{
				return (double)(pointRecognitionAngle[i]);
			}
		}
		public double returnRecognitionAll(int i)
		{
			lock(this)
			{
				return (double)pointRecognitionAll[i];
			}
		}

		public void addRecognitionSpeed(double speed) //dodamo hitrost
		{
			lock(this)
			{
				pointRecognitionSpeed.Add(speed);
			}
		}

		public double returnRecognitionSpeed(int i)
		{
			lock(this)
			{
				return (double)pointRecognitionSpeed[i];
			}
		}
		
		public void addRecognitionAcc(double acc)  //dodamo pospeek
		{
			lock(this)
			{
				pointRecognitionAcc.Add(acc);
			}
		}

		public double returnRecognitionAcc(int i)
		{
			lock(this)
			{
				return (double)pointRecognitionAcc[i];
			}
		}

		public void clearPointRecognition()  //spucamo seznam  prepoznav
		{
			lock(this)
			{
				pointRecognition.Clear();
			}
		}
		public void clearPointRecognitionSpeed() //spucamo seznam hitrosti v tockah
		{
			lock(this)
			{
				pointRecognitionSpeed.Clear();
			}
		}
		public void clearPointRecognitionAngle()  //spucamo seznam kotov
		{
			lock(this)
			{
				pointRecognitionAngle.Clear();
			}
		}
		public void clearPointRecognitionAll()
		{
			lock(this)
			{
				pointRecognitionAll.Clear();
			}
		}
		public Point getPoint() 
		{
			lock(this)
			{
				return tockaK;
			}
		}
		public void setPoint(Point p)
		{
			lock(this)
			{
				tockaK=p;
			}
	    }


		//--------------------------------------------------------------------

		
		public void addColor(Color c)		//dodamo barvo 
		{
			lock(this)
			{
				strokeColor = c;
			}
		}
	
		public void addWidth(int w)			//dodamo sirino
		{
			lock(this)
			{
				width = w;
			}
		}

		public int numberOfPoints()			//vrenomo stevilo tock
		{
			lock(this)
			{
				return oneStroke.Count;
			}
		}
		public int numberOfPointsA()
		{
			lock(this)
			{
				return pointRecognitionAngle.Count;
			}
		}


		public Point returnElement(int i)		//vrnemo elemnt i iz tabele
		{
			lock(this)
			{
				return (Point)oneStroke[i];
			}
	
		}

		public double returnTimeOfElement(int i)
		{
			lock(this)
			{
				return (double)time_intervals[i];
			}
		}

		public double returnRecognition(int i)
		{
			lock(this)
			{
				return (double)pointRecognition[i];
			}
		}


		public Color returnColor()
		{
			lock(this)
			{
				return (Color)strokeColor;
			}
		}

		public int returnWidth()
		{
			lock(this)
			{
				return width;
			}
		}

		public void changeRecognition(int i,double newRecog)		//spremenimo prepoznavo
		{
			lock(this)
			{
				pointRecognition.RemoveAt(i);
				pointRecognition.Insert(i,newRecog);
			}

		}

		public void removePoint(int index)		//odstranimo tocko na mestu "index" v tabeli 
		{
			lock(this)
			{
				if(index >= 0 && index<oneStroke.Count) 
				oneStroke.RemoveAt(index);
			}
		
		}

	

		public void clearStroke()						//spucamo vse  sezname v razredu 
		{
			lock(this)
			{
				oneStroke.Clear();
				time_intervals.Clear();
				pointRecognition.Clear();
				pointRecognitionAngle.Clear();
				pointRecognitionSpeed.Clear();
				pointRecognitionAcc.Clear();
				pointForce.Clear();
				forceSim.clear();
				imenPovezav.Clear();
				inkiPovezav.Clear();
				vzporednePovezave.Clear();
				sosednjiStroki.Clear();
				
				width = 0;
			}
		}

		public void spremeniVrstoObjekta(bool vrsta)
		{
			lock(this)
			{
				sklenjenObjekt = vrsta;
			}
		}

		public bool vrniVrsto()
		{
			lock(this)
			{
				return sklenjenObjekt;
			}
		}



//--------------------------------------------SILE---------------------------------------------------
		public void addForceItem(ForceItem forItm)
		{
			lock(this)
			{
				pointForce.Add(forItm);
				forceSim.addItem(forItm);
			}
		}

		public void addForce(Force force)
		{
			lock(this)
			{
			
				forceSim.addForce(force);
			}
		}

		public void addSpring(ForceItem fi1,ForceItem fi2, float coeff, float length)
		{
			lock(this)
			{
				forceSim.addSpring(fi1,fi2);
			}
		}

		public void addSpeedlimit(float speed)
		{
			lock(this)
			{
				forceSim.SpeedLimit = speed;
			}
		}
	/*	public void removeSpring()
		{
			forceSim.Springs
			forceSim.removeSpring(asdsd,);
		}*/


		public void removeForceItem(int index)		//odstranimo telo  na mestu "index"  
		{
			lock(this)
			{
				forceSim.removeItem((ForceItem)pointForce[index]);
				pointForce.RemoveAt(index);
			}
		}

		
		public void changePointForce(Point p,int index)		//spremenimo kordinate tocke na mestu index za sile
		{
			lock(this)
			{
				((ForceItem)pointForce[index]).location[0]= p.X;
				((ForceItem)pointForce[index]).location[1]= p.Y;
			}
		}

		public void changePointsForces(Point p,ArrayList indeksi)		//spremenimo kordinate vecih tock,ki v bistvu sestavljajo eno tocko
		{
			lock(this)
			{
				int prvatock = (int)indeksi[0];
				IEnumerator ien = indeksi.GetEnumerator();
		
				((ForceItem)pointForce[prvatock]).location[0]= p.X;
				((ForceItem)pointForce[prvatock]).location[1]= p.Y;
			
				ien.MoveNext();
				while(ien.MoveNext())
				{
					this.removeForceItem((int)ien.Current);
					pointForce.Insert((int)ien.Current,pointForce[prvatock]);
					forceSim.addItem((ForceItem)pointForce[prvatock]);
				}
			}
			
		}


		public ForceItem returnForceItem(int index)
		{
			lock(this)
			{
				return (ForceItem)pointForce[index];
			}
		}

		public IEnumerator returnForceIEnumerator()
		{
			lock(this)
			{
				return pointForce.GetEnumerator();
			}
		}

		public ForceSimulator returnForceSimulator()
		{
			lock(this)
			{
				return forceSim;
			}
		}

			public void runForceSimulator(long timestep,int pen_width,Graphics g)
			{
				lock(this)
				{
					Pen pp = new Pen(Color.White,pen_width);
					Recognizer2 recog2 = new Recognizer2();
					int velikost = Form1.razdaljaMedStikalisciTock;
				
				
					forceSim.runSimulator(timestep);

					for(int i = 0;i<oneStroke.Count;i++)
					{
						Point p = new Point((int)((ForceItem)pointForce[i]).location[0],
							(int)((ForceItem)pointForce[i]).location[1]);
				
						if(recog2.pointDistance(p,(Point)oneStroke[i]) > 10)
						{
						
							//spremenjenaTocka.Add(true);
							if(i == 0)
							{
								g.DrawEllipse(pp,((Point)oneStroke[i]).X-velikost/2,((Point)oneStroke[i]).Y-velikost/2,velikost,velikost);
								g.DrawEllipse(pp,((Point)oneStroke[i+1]).X-velikost/2,((Point)oneStroke[i+1]).Y-velikost/2,velikost,velikost);
								g.DrawLine(pp,(Point)oneStroke[i],(Point)oneStroke[i+1]);
							}
							else if(i > 0 && i < (oneStroke.Count-1))
							{
								g.DrawEllipse(pp,((Point)oneStroke[i]).X-velikost/2,((Point)oneStroke[i]).Y-velikost/2,velikost,velikost);
								g.DrawEllipse(pp,((Point)oneStroke[i-1]).X-velikost/2,((Point)oneStroke[i-1]).Y-velikost/2,velikost,velikost);
								g.DrawEllipse(pp,((Point)oneStroke[i+1]).X-velikost/2,((Point)oneStroke[i+1]).Y-velikost/2,velikost,velikost);
								g.DrawLine(pp,(Point)oneStroke[i],(Point)oneStroke[i+1]);
								g.DrawLine(pp,(Point)oneStroke[i-1],(Point)oneStroke[i]);
							}
							else if( i == (oneStroke.Count-1))
							{
								g.DrawEllipse(pp,((Point)oneStroke[i-1]).X-velikost/2,((Point)oneStroke[i-1]).Y-velikost/2,velikost,velikost);
								g.DrawEllipse(pp,((Point)oneStroke[i]).X-velikost/2,((Point)oneStroke[i]).Y-velikost/2,velikost,velikost);
								g.DrawLine(pp,((Point)oneStroke[i-1]),((Point)oneStroke[i]));
							}
							//if(recog2.pointDistance(p,(Point)oneStroke[i]) > 11)
							this.changePoint(p,i);
						}
					}
				}
			}

	}
/*-----------------------------------------------------------------------------------------------------------*/
	public class allObjects				//vsi objetki, ki jih vsebuje panel
	{
		
		private ArrayList objectsList =  new ArrayList();

		public allObjects()
		{
			//
			// TODO: Add constructor logic here
			//
			
		}
        
				
		~allObjects()
		{
			objectsList.Clear();
		}

		public void  addObject(Object1 l)
		{
			objectsList.Add(l);
		}

		public Object1 returnObject(int i)
		{
			return (Object1)objectsList[i];
		}

		public int returnNumberOfObjects()
		{
			return objectsList.Count;
		}

		public void removeLast()
		{
			Object1 obj;
			obj = (Object1)objectsList[objectsList.Count-1];
			obj.removeAll();
			objectsList.RemoveAt(objectsList.Count-1);
		}
		
		public void removeAll()
		{
			Object1 obj;
			for(int i = 0;i < objectsList.Count;i++)
			{
				obj = (Object1)objectsList[i];
				obj.removeAll();
			}


			objectsList.Clear();
		}


	}
/*-----------------------------------------------------------------------------------------------------------*/
	public class Object1    //objekt predstavljen z premicami
	{
		
		private ArrayList lineList =  new ArrayList();

		
		public Object1()
		{
			//
			// TODO: Add constructor logic here
			//
			
		}
        
				
		~Object1()
		{
			lineList.Clear();
		}

		public void  addLine(Line l)
		{
			lineList.Add(l);
		}

		public Line returnLine(int i)
		{
			return (Line)lineList[i];
		}

		public int returnNumberOfLines()
		{
			return lineList.Count;
		}

		public void removeLast()
		{
				lineList.RemoveAt(lineList.Count-1);
		}
		
		public void removeAll()
		{
			lineList.Clear();
		}

	}


/*-----------------------------------------------------------------------------------------------------------*/
	public class Line   //razred v katerega shranimo eno premico 
	{
		
		private Point p1  = Point.Empty;
		private Point p2 = Point.Empty;
		private double a = 0,n = 0;				// y = a*x +n enacba  za premico 
		private int upX = 0,upY = 0,downX = 0, downY = 0;		//meje premice,ki smo jo narisali
		private string color ;
		private int width;

		
		public Line()
		{
			//
			// TODO: Add constructor logic here
			//
			
		}
		
		
		~Line()
		{
			p1.X = 0;
			p1.Y = 0;
			p2.X = 0;
			p2.Y = 0;
		}

		public void addFirstPoint(int x,int y)
		{
				p1 = new Point(x,y);
		}



		public void addLastPoint(int x,int y)
		{
				p2 = new Point(x,y);
		}

		public void addFirstPoint(Point p3)
		{
			p1 = p3;
		}


		public void addLastPoint(Point p4)
		{
			p2 = p4;
		}
		
		public void addColor(string s)
		{
			color = s;
		}

		public void addWidth(int w)
		{
			width = w;

		}
 
		public Point returnFirstPoint()
		{
				return p1;
		}

		public Point returnLastPoint()
		{
				return p2;
		}

		public string returnColor()
		{
			return color;
		}

		public int returnWidth()
		{
			return width;
		}

		public void moveLine(int x, int y)		//premaknemo premico za x in y
		{
			p1.X = p1.X + x;
			p2.X = p2.X + x;
			p1.Y = p1.Y + y;
			p2.Y = p2.Y + y;
            
		}

		public void moveLine(Point p3)		//premaknemo premico za tocko p3
		{
			p1.X = p1.X + p3.X;
			p2.X = p2.X + p3.X;
			p1.Y = p1.Y + p3.Y;
			p2.Y = p2.Y + p3.Y;
            
		}

		public void  setSize(int length)	//nastavimo velikost premice se ne dela cist dobr
		{
				int leng;
            			
				leng  =(int)Math.Sqrt(Math.Pow(p2.X-p1.X,2)+ Math.Pow(p1.Y-p2.Y,2));

			while(leng > length)
			{
				
				if(p2.X > p1.X)
				{
					p1.X++;
					p1.Y--;
					p2.X--;
					p2.Y++;

				}
				else
				{
					p1.X--;
					p1.Y--;
					p2.X++;
					p2.Y++;
				}
				leng  =(int)Math.Sqrt(Math.Pow(p2.X-p1.X,2)+ Math.Pow(p1.Y-p2.Y,2));
			}
	
		}

		public void CreateLine()			//poiscemo koeficienta premice in meje s katerimi je omejena
		{
			n = (p1.Y*p2.X)/(p2.X-p1.X)-(p1.X*p2.Y)/(p2.X-p1.X);
			a = (p1.Y - n)/p1.X;

			if(p1.X >= p2.X)
			{
				upX = p1.X;
				downX = p2.X;
			}
			else 
			{
				upX = p2.X;
				downX = p1.X;
			}

			if(p1.Y >= p2.Y)
			{
				upY = p1.Y;
				downY = p2.Y;
			}
			else 
			{
				upY = p2.Y;
				downY = p1.Y;
			}

		}

		public bool isPointOnLine(Point po)//ali je tocka na ekranu na nasi crti
		{
			double y;
			if(po.X <= upX && po.X >= downX && po.Y <= upY && po.Y >= downY)
			{
				y = (int)(a*po.X +n);
				if(Math.Abs(y-po.Y) <= 5)    //tukaj dolocimo koliko lahko odstopa tocka od premice 
					return true;
				else
					return false;
			}
			else
				return false;
			
		
		}

		public int distancToPoint(Point p3) //razdalja tocke do premice 
		{

            int rez = 0;
						
	
			rez = (int)(Math.Abs((p2.X-p1.X)*(p1.Y-p3.Y)-(p1.X-p3.X)*(p2.Y-p1.Y))/
					Math.Sqrt(Math.Pow(p2.X-p1.X,2)+Math.Pow(p2.Y-p1.Y,2)));


		
           	return rez;

		}

	}

/*-------------------------------------------------------------------------------------------------------------*/
	
}
