using System;
namespace symmetry
{
	
	/// <summary> Updates velocity and position data using Euler's Method. This is a
	/// simple and fast, though somewhat inaccurate, approach.
	/// 
	/// </summary>
	/// <version>  1.0
	/// </version>
	/// <author>  <a href="http://jheer.org">Jeffrey Heer</a> prefuse(AT)jheer.org
	/// </author>
	public class EulerIntegrator : Integrator
	{
		
		/// <seealso cref="long)">
		/// </seealso>
		public virtual void  integrate(ForceSimulator sim, long timestep)
		{
			float speedLimit = sim.SpeedLimit;
			System.Collections.IEnumerator iter = sim.Items;
			//UPGRADE_TODO: Method 'java.util.Iterator.hasNext' was converted to 'System.Collections.IEnumerator.MoveNext' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratorhasNext"'
			while (iter.MoveNext())
			{
				//UPGRADE_TODO: Method 'java.util.Iterator.next' was converted to 'System.Collections.IEnumerator.Current' which has a different behavior. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1073_javautilIteratornext"'
				ForceItem item = (ForceItem) iter.Current;
				item.location[0] += timestep * item.velocity[0];
				item.location[1] += timestep * item.velocity[1];
				float coeff = timestep / item.mass;
				item.velocity[0] += coeff * item.force[0];
				item.velocity[1] += coeff * item.force[1];
				float vx = item.velocity[0];
				float vy = item.velocity[1];
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				float v = (float) System.Math.Sqrt(vx * vx + vy * vy);
				if (v > speedLimit)
				{
					item.velocity[0] = speedLimit * vx / v;
					item.velocity[1] = speedLimit * vy / v;
				}
			}
		} //
	} // end of class EulerIntegrator
}