using System;
namespace symmetry
{
	
	/// <summary> Uses a gravitational force model to act as a "wall". Can be used to
	/// construct line segments which either attract or repel items.
	/// 
	/// </summary>
	/// <version>  1.0
	/// </version>
	/// <author>  <a href="http://jheer.org">Jeffrey Heer</a> prefuse(AT)jheer.org
	/// </author>
	public class WallForce:AbstractForce
	{
		override public bool ItemForce
		{
			get
			{
				return true;
			}
			//
			
		}
		override protected internal System.String[] ParameterNames
		{
			get
			{
				return pnames;
			}
			//
			
		}
		
		private static System.String[] pnames = new System.String[]{"GravitationalConstant"};
		
		public const float DEFAULT_GRAV_CONSTANT = - 0.1f;
		public const int GRAVITATIONAL_CONST = 0;
		
		private float x1, y1, x2, y2;
		private float dx, dy;
		
		public WallForce(float gravConst, float x1, float y1, float x2, float y2)
		{
			params_Renamed = new float[]{gravConst};
			this.x1 = x1; this.y1 = y1;
			this.x2 = x2; this.y2 = y2;
			dx = x2 - x1;
			dy = y2 - y1;
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			float r = (float) System.Math.Sqrt(dx * dx + dy * dy);
			if (dx != 0.0)
				dx /= r;
			if (dy != 0.0)
				dy /= r;
		} //
		
		public WallForce(float x1, float y1, float x2, float y2):this(DEFAULT_GRAV_CONSTANT, x1, y1, x2, y2)
		{
		} //
		
		/* (non-Javadoc)
		* @see edu.berkeley.guir.prefusex.force.Force#getForce(edu.berkeley.guir.prefusex.force.ForceItem)
		*/

		public double distancToPoint(float x1,float y1,float x2,float y2,float n0, float n1) //razdalja tocke do premice 
		{

			double rez = 0;
						
	
			rez = (int)(Math.Abs((x2-x1)*(y1-n1)-(x1-n0)*(y2-y1))/
				Math.Sqrt(Math.Pow(x2-x1,2)+Math.Pow(y2-y1,2)));


		
			return rez;

		}

		
		public double relativeCCW(float x1,float y1,float x2,float y2,float n0, float n1) //razdalja tocke do premice 
		{

			double rez = 0;
						
	
			rez = (int)(Math.Abs((x2-x1)*(y1-n1)-(x1-n0)*(y2-y1))/
				Math.Sqrt(Math.Pow(x2-x1,2)+Math.Pow(y2-y1,2)));

			return rez;

		}




		public override void  getForce(ForceItem item)
		{
					
			float[] n = item.location;
			//UPGRADE_TODO: Method 'java.awt.geom.Line2D.relativeCCW' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			int ccw = relativeCCW(x1, y1, x2, y2, n[0], n[1]);
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			//UPGRADE_TODO: Method 'java.awt.geom.Line2D.ptSegDist' was not converted. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1095"'
			float r = (float) distancToPoint(x1, y1, x2, y2, n[0], n[1]);
			if (r == 0.0)
			{
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				r = (float) SupportClass.Random.NextDouble() / 100.0f;
			}
			float v = params_Renamed[GRAVITATIONAL_CONST] * item.mass / (r * r * r);
			if (n[0] >= System.Math.Min(x1, x2) && n[0] <= System.Math.Max(x1, x2))
				item.force[1] += ccw * v * dx;
			if (n[1] >= System.Math.Min(y1, y2) && n[1] <= System.Math.Max(y1, y2))
				item.force[0] += (- 1) * ccw * v * dy;
		} //
	} // end of class WallForce
}